VARIABLES_INFO_ATTR_NAME = '_eaas_variable_info'
EVENT_INFO_ATTR_NAME = '_eaas_event_info'
SCHEDULABLE_INFO_ATTR_NAME = '_eaas_schedulable_info'
ANVIL_KEY_VAR_ATTR_NAME = '_eaas_anvil_key_var'
ANVIL_CALLABLE_ATTR_NAME = '_eaas_anvil_callable'
GUEST_ENDPOINT_ATTR_NAME = '_eaas_guest_endpoint'
ACCOUNT_SETTINGS_PAGE_ATTR_NAME = '_eaas_account_settings_page'
MODULE_PAGES_ATTR_NAME = '_eaas_module_pages'
ADMIN_PAGES_ATTR_NAME = '_eaas_admin_pages'
